import React, { Component } from 'react';
import { expect } from 'chai';
import { spy, stub } from 'sinon';
import { shallow } from 'enzyme';

import { Route } from 'react-router-dom';
import { UniversalRouter } from './UniversalRouter';

const routes = [
  {
    path: '/foo',
    //  eslint-disable-next-line
    component: class Foo extends Component {
      static getInitialProps = stub().returns(Promise.resolve('foo!'))
      render() {
        return (<div>Foo route</div>);
      }
    },
    isAuth: true,
  },
  {
    path: '/bar',
    //  eslint-disable-next-line
    component: class Bar extends Component {
      static getInitialProps = stub().returns(Promise.resolve('bar!'))
      static cache = true
      render() {
        return (<div>Bar route</div>);
      }
    },
    isAuth: false,
  },
  {
    path: '/wee',
    component: () => (<div>Wee route</div>),
    isAuth: false,
  },
];

const defaultProps = {
  location: { pathname: '/foo' },
  history: { listen: spy() },
  routes,
  serverProps: {
    '/foo': {},
  },
};

describe('UniversalRouter', () => {
  it('should create a Route and pass props to it', () => {
    const wrapper = shallow(<UniversalRouter {...defaultProps} />);

    wrapper.find(Route).forEach((route, index) => {
      expect(route.props().path).to.equal(routes[index].path);
      expect(route.props().isAuth).to.equal(routes[index].isAuth);
    });
  });

  it('should set initial state to value of serverProps', () => {
    const wrapper = shallow(<UniversalRouter {...defaultProps} />);

    expect(wrapper.state()).to.eql(defaultProps.serverProps);
  });

  describe('Getting initialProps', () => {
    it('should attempt to resolve initialProps when mounted', async () => {
      const wrapper = shallow(<UniversalRouter {...defaultProps} />);
      const instance = wrapper.instance();
      instance.componentDidMount();
      await Promise.resolve();

      expect(wrapper.state()['/foo']).to.equal('foo!');
    });

    it('should not get initialProps if route cache is true', async () => {
      defaultProps.location.pathname = '/bar';
      const wrapper = shallow(<UniversalRouter {...defaultProps} />);
      const instance = wrapper.instance();
      instance.componentDidMount();
      await Promise.resolve();

      expect(wrapper.state()['/bar']).to.equal('bar!');
    });
  });
});
