import React, { Component } from 'react';
import PropTypes from 'prop-types';
import { Route, Switch, withRouter } from 'react-router-dom';

import renderRoute from './renderRoute';
import { matchRoute, resolveInitialProps } from './../utils';

export class UniversalRouter extends Component {
  static contextTypes = { router: PropTypes.any }
  static childContextTypes = { getRouteData: PropTypes.func.isRequired }

  constructor(props) {
    super(props);
    const { serverProps } = props;
    this.state = { ...serverProps };
  }

  getChildContext = () => ({
    getRouteData: path => (this.state[path]),
  })

  componentDidMount() {
    const { location, history } = this.props;

    this.fetchInitialProps(location);
    history.listen(this.fetchInitialProps);
  }

  fetchInitialProps = async ({ pathname }) => {
    const { path, component } = matchRoute(this.props.routes, pathname);
    const cache = Boolean(component.cache);

    if (!cache || !this.state.hasOwnProperty(pathname)) {
      const data = await resolveInitialProps(component, path, pathname);
      this.setState(Object.assign(this.state, { [pathname]: data }));
    }
  }

  render() {
    const { routes } = this.props;

    return (
      <Switch>
        {routes.map(({ path, component, ...rest }) => (
          <Route
            key={path}
            path={path}
            component={renderRoute(component)}
            {...rest}
          />
        ))}
      </Switch>
    );
  }
}

export default withRouter(UniversalRouter);
